/* 
    TVscreen Widget for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * TVscreen.c 
 */


#include "TVscreenP.h"

/*#define DEBUG*/

#define DEFAULT_WIDTH    320
#define DEFAULT_HEIGHT   240  

#define PAL_WIDTH 768
#define PAL_HEIGHT 576

#define NTSC_WIDTH 640
#define NTSC_HEIGHT 480

#define DEFAULT_DELAY 100

#define offset(field) XtOffsetOf(TVscreenRec, field)


static XtResource resources[] = {
  {
    XtNdevName, 
    XtCDevName, 
    XtRString, 
    sizeof(String),
    offset(TVscreen.devname), 
    XtRImmediate, 
    NULL
  },
  {
    XtNcap, 
    XtCCap, 
    XtRBoolean, 
    sizeof(Boolean),
    offset(TVscreen.CAP_ON), 
    XtRImmediate, 
    (XtPointer) False
  },
  {
    XtNnorm, 
    XtCNorm, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.NORM), 
    XtRImmediate, 
    (XtPointer) DNORM
  },
  {
    XtNfreq, 
    XtCFreq, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.FREQ), 
    XtRImmediate, 
    (XtPointer) 0
  },
  {
    XtNinputSelect, 
    XtCInputSelect, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.INPUT), 
    XtRImmediate, 
    (XtPointer) DIN
  },
  {
    XtNdelay, 
    XtCDelay, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.DELAY), 
    XtRImmediate, 
    (XtPointer) DEFAULT_DELAY
  },
  {
    XtNbright, 
    XtCBright, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.Bright), 
    XtRImmediate, 
    (XtPointer) DBRI
  },
  {
    XtNcol, 
    XtCCol, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.Color), 
    XtRImmediate, 
    (XtPointer) DCOL
  },
  {
    XtNhue, 
    XtCHue, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.Hue), 
    XtRImmediate, 
    (XtPointer) DHUE
  },
  {
    XtNcontrast, 
    XtCContrast, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.Contrast), 
    XtRImmediate, 
    (XtPointer) DCONTRAST
  },
  {
    XtNrefresh, 
    XtCRefresh, 
    XtRBoolean, 
    sizeof(Boolean),
    offset(TVscreen.REFRESH), 
    XtRImmediate, 
    (XtPointer) True
  },
  {
    XtNbpp, 
    XtCBpp, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.bpp), 
    XtRImmediate, 
    (XtPointer) 0
  },
  {
    XtNvis, 
    XtCVis, 
    XtRString, 
    sizeof(String),
    offset(TVscreen.visname), 
    XtRImmediate, 
    NULL
  },
  {
    XtNsnap, 
    XtCSnap, 
    XtRBoolean, 
    sizeof(Boolean),
    offset(TVscreen.snap), 
    XtRImmediate, 
    (XtPointer) False
  },
  {
    XtNrealW, 
    XtCRealW, 
    XtRInt, 
    sizeof(int),
    offset(TVscreen.realw), 
    XtRImmediate, 
    (XtPointer) 0
  },
};

/* Declaration of methods */

static void Initialize();
static void Realize();
static Boolean SetValues();
static void destroy();
static XtGeometryResult QueryGeometry();
static int rx,ry,rh,rw;

/* the following are actions of TVscreen */
static void pConfig();
static void addclip();
static void maketab();
static int getclip();
static void bt848_dma();
static void SetGeo();
static void cap();
static void setmaxsize();
static void ref();
static void refresh();
XErrorHandler xerror();

static Window lastrwin;
static XtIntervalId timer;
static Boolean Move=False;

TVscreenClassRec tvscreenClassRec = {
    {
    /* core_class fields */
    /* superclass        */ (WidgetClass) &coreClassRec,
    /* class_name        */ "TVscreen",
    /* widget_size       */ sizeof(TVscreenRec),
    /* class_initialize      */ NULL,
    /* class_part_initialize     */ NULL,
    /* class_inited          */ FALSE,
    /* initialize        */ Initialize,
    /* initialize_hook       */ NULL,
    /* realize           */ Realize,
    /* actions           */ NULL,
    /* num_actions       */ 0,
    /* resources         */ resources,
    /* num_resources         */ XtNumber(resources),
    /* xrm_class         */ NULLQUARK,
    /* compress_motion       */ TRUE,
    /* compress_exposure     */ XtExposeCompressMultiple,
    /* compress_enterleave   */ TRUE,
    /* visible_interest      */ FALSE,
    /* destroy           */ destroy,
    /* resize            */ NULL,
    /* expose            */ NULL,
    /* set_values        */ SetValues,
    /* set_values_hook       */ NULL,
    /* set_values_almost     */ XtInheritSetValuesAlmost,
    /* get_values_hook       */ NULL,
    /* accept_focus      */ NULL,
    /* version           */ XtVersion,
    /* callback_private      */ NULL,
    /* tm_table          */ NULL,
    /* query_geometry        */ QueryGeometry,
    /* display_accelerator       */ XtInheritDisplayAccelerator,
    /* extension                 */ NULL
    },
    {
    /* extension          */        0,
    },
};

WidgetClass tvscreenWidgetClass = (WidgetClass) & tvscreenClassRec;

/* Realize */
static void Realize(Widget w,
		    XtValueMask *value_mask,
		    XSetWindowAttributes *attributes)
{
  TVscreenWidget tw=(TVscreenWidget) w;
    
  XtCreateWindow(w, (unsigned int) InputOutput,
		 tw->TVscreen.visual, *value_mask, attributes);

  if (tw->TVscreen.CAP_ON){
    tw->TVscreen.Change=TRUE;
#ifndef NOCARD
    ioctl(tw->TVscreen.fbttv, BTTV_SETWTW, &(tw->TVscreen.wtw));     
    ioctl(tw->TVscreen.fbttv, BTTV_SETFREQ, &tw->TVscreen.wtw.freq);
    ioctl(tw->TVscreen.fbttv, BTTV_INPUT, &(tw->TVscreen.INPUT));
#endif 
 }
} 


XErrorHandler xerror(Display *dpy, XErrorEvent *event)
{
  switch (event->error_code){
  case BadWindow:
#ifdef DEBUG
    printf("Bad Window ingnored\n");
#endif
    return 0 ;
    break;
      
  case BadMatch:
#ifdef DEBUG
    printf("BadMatch ingnored\n");
#endif
    return 0 ;
    break;

  case BadColor:
#ifdef DEBUG
    printf("BadColor ingnored\n");
#endif
    return 0 ;
    break;

    default:
      return 0;
  }
}

void ref(TVscreenWidget w)
{
#ifndef NOCARD
  int scr,depth;
  XSetWindowAttributes xswa;
  unsigned long mask;
  Window rwin,rootW;
  Display *disp;
  
  if (w->TVscreen.REFRESH){
    disp=XtDisplay((Widget)w);
    scr=DefaultScreen(disp);
    rootW= RootWindow(disp, scr);
    depth=w->core.depth;
    
    xswa.override_redirect = True;
    xswa.backing_store = NotUseful;
    xswa.save_under = False;
    xswa.background_pixel=BlackPixel(disp,scr);
    xswa.backing_store=NotUseful;
    mask = (CWBackPixel | CWSaveUnder | CWBackingStore|
	    CWOverrideRedirect );
    if(Move){
      rwin = XCreateWindow(disp,rootW,0, 0, 
			   w->TVscreen.wtw.swidth,w->TVscreen.wtw.sheight, 0,
			   CopyFromParent, InputOutput, CopyFromParent,
			   mask, &xswa);
      Move=False;
    } else {
      rwin = XCreateWindow(disp,rootW,
			   w->TVscreen.wtw.x-80, w->TVscreen.wtw.y-80, 
			   w->TVscreen.wtw.width+80,w->TVscreen.wtw.height+80,
			   0,CopyFromParent, InputOutput, CopyFromParent,
			   mask, &xswa);
    }
    lastrwin=rwin;
    XMapWindow(disp, rwin);
    XUnmapWindow(disp, rwin);
    XDestroyWindow(disp, rwin);
  }
#endif
}

 
static void refresh(w)
TVscreenWidget w;
{
  if (timer) XtRemoveTimeOut(timer);

  timer=XtAppAddTimeOut(XtWidgetToApplicationContext((Widget)w),
			w->TVscreen.DELAY,(XtTimerCallbackProc)ref,w);
}


#ifndef NOCARD
static void addclip(cr,ncr,x,y,x2,y2)
struct cliprec *cr;
int *ncr; 
uint x,y,x2,y2;
{
  cr[*ncr].x=x;
  cr[*ncr].y=y;
  cr[*ncr].x2=x2;
  cr[*ncr].y2=y2;
  *ncr=*ncr+1;
} 
#endif

static void maketab(w, ncr)
     TVscreenWidget w;
     int ncr;
{
#ifndef NOCARD
  w->TVscreen.cliprecs[MAX_CLIPRECS-1].x=ncr;
  ioctl(w->TVscreen.fbttv, BTTV_SETCLIP, w->TVscreen.cliprecs);
#endif
}

#ifndef NOCARD
static int getclip(w)
TVscreenWidget w;
{
  int ncr=0;
  int x,y,x2,y2,wx,wy;
  uint ww,wh;
  struct bttv_window wtw;
  Display *disp;
  XWindowAttributes wts;
  Window parent,win,parent2, root, rroot, *children, swin;
  uint nchildren, i;
  struct cliprec *cr; 
  Widget widpar;


  if (!XtIsRealized((Widget)w)) return;

  disp=XtDisplay((Widget)w);
  wtw=w->TVscreen.wtw;

  widpar=(Widget) w;
  while(XtParent(widpar)) widpar=XtParent(widpar);
  if((win=XtWindow(widpar))==None) return;

  XQueryTree(disp,win, &rroot, &parent, &children, &nchildren);
  if(nchildren) XFree((char *)children);

  cr=w->TVscreen.cliprecs;

  wx=wtw.x; wy=wtw.y;
  wh=wtw.height; ww=wtw.width;
  if (wx<0)
    addclip(cr, &ncr, 0, 0, (uint)(-wx), wh-1);
  if (wy<0)
    addclip(cr, &ncr, 0, 0, ww-1, (uint)(-wy));
  if ((wx+ww) > wtw.swidth)
    addclip(cr, &ncr, wtw.swidth-wx, 0, ww-1, wh-1);
  if ((wy+wh) > wtw.sheight)
    addclip(cr, &ncr, 0, wtw.sheight-wy, ww-1, wh-1);
    
  root=DefaultRootWindow(disp);
  swin=win;
  while (parent!=root) {
    swin=parent;
    XQueryTree(disp, swin, &rroot, &parent, &children, &nchildren);
    if (nchildren)
      XFree((char *) children);
  }
  XQueryTree(disp, root, &rroot, &parent2, &children, &nchildren);
  for (i=0; i<nchildren;i++) 
    if (children[i]==swin) break;
  i++;
  for (; i<nchildren; i++) {
    
    XGetWindowAttributes(disp, children[i], &wts);
    if (!(wts.map_state & IsViewable))  continue;

    x=wts.x-wtw.x; y=wts.y-wtw.y;
    x2=x+wts.width+2*wts.border_width-1;
    y2=y+wts.height+2*wts.border_width-1;
    if ((x2 < 0) || (x >= (int)ww) ||
	(y2 < 0) || (y >= (int)wh))
      continue;

    if (x<0)
      x=0;
    if (y<0)
      y=0;
    if (x2>=(int)ww)
      x2=ww-1;
    if (y2>=(int)wh)
      y2=wh-1;
    addclip(cr, &ncr, x, y, x2, y2);
  }
  XFree((char *) children);


  maketab(w,ncr);
  return ncr;
}
#endif

#ifndef NOCARD
static Boolean onscreen(w,wts)
TVscreenWidget w;
XWindowAttributes wts;
{
  int x,x2,y,y2;
  uint ww,wh;
  struct bttv_window wtw;
  
  wtw=w->TVscreen.wtw;
  wh=wtw.height; ww=wtw.width;

  if (!(wts.map_state & IsViewable)) return False;

  x=wts.x-wtw.x; y=wts.y-wtw.y;
  x2=x+wts.width+2*wts.border_width-1;
  y2=y+wts.height+2*wts.border_width-1;
  if ((x2 < 0) || (x >= (int)ww) ||
      (y2 < 0) || (y >= (int)wh))
    return False;
  return True;
}
#endif

static void bt848_dma(w,data)
Widget w;
uint data;
{
#ifndef NOCARD
  int f;
  TVscreenWidget cw = (TVscreenWidget) w;
  
  f=cw->TVscreen.fbttv;
  ioctl(f, BTTV_DMA, &data);
#endif
}  

static void cap(w,W)
TVscreenWidget w;
int W;
{
  if( XtIsRealized((Widget)w)) {
#ifndef NOCARD
    if(W && w->TVscreen.CAP_ON) ioctl(w->TVscreen.fbttv, BTTV_CAP_ON, NULL);
    else ioctl(w->TVscreen.fbttv, BTTV_CAP_OFF, NULL);
#endif
  }
}

static void setmaxsize(cw)
TVscreenWidget cw;
{
  uint max_h,max_w;

  switch (cw->TVscreen.NORM){
  case PAL:
  case SECAM:
    max_w=PAL_WIDTH;
    max_h=PAL_HEIGHT;
    break;

  case NTSC:
    max_w=NTSC_WIDTH;
    max_h=NTSC_HEIGHT;
    break;
    
  default:
    max_w=DEFAULT_WIDTH;
    max_h=DEFAULT_HEIGHT;
    break;
  }

  cw->TVscreen.max_width=max_w;
  cw->TVscreen.max_height=max_h;
}
  
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
  Widget parent;
  char *bttvname;
  Screen *screen;
  Window rootw;
  int col;
  uint rwidth; 
  TVscreenWidget new = (TVscreenWidget) tnew;
  int major, minor, bank, ram;
  Display *disp;
  ushort i,r,rr;
  Colormap cmap;
  XColor *my;
  Visual *v;
  unchar *tm;
  int msize,ii;

#ifndef NOCARD
  if(new->TVscreen.devname) bttvname=new->TVscreen.devname;
  else bttvname="/dev/bttv";

  new->TVscreen.devname=bttvname;

  if ((new->TVscreen.fbttv=open(bttvname,O_RDWR)) < 0) {
    printf("Cannot open %s\n",bttvname);
    exit(1);
  }

  ioctl(new->TVscreen.fbttv, BTTV_GETWTW, &(new->TVscreen.wtw));
  new->TVscreen.wtw.width=DEFAULT_WIDTH;
  new->TVscreen.wtw.height=DEFAULT_HEIGHT;
  new->TVscreen.wtw.x=0;
  new->TVscreen.wtw.y=0;
  new->TVscreen.wtw.cropwidth=768;
  new->TVscreen.wtw.cropheight=576;
  new->TVscreen.wtw.cropx=0;
  new->TVscreen.wtw.cropy=0;
  new->TVscreen.wtw.swidth=XtScreen(tnew)->width;
  new->TVscreen.wtw.sheight=XtScreen(tnew)->height;


  rwidth=XtScreen(tnew)->width;
  disp=XtDisplay(tnew);
#ifndef NODGA
  if (XF86DGAQueryVersion(disp, &major, &minor)) 
    XF86DGAGetVideoLL(disp, DefaultScreen(disp), &(new->TVscreen.wtw.vidadr),
		      &rwidth, &bank, &ram);
  else 
#endif
#endif
    if (rwidth<960)
      if (rwidth>640)
	rwidth=960;
  if (new->TVscreen.realw) rwidth=new->TVscreen.realw;
  
  new->TVscreen.dual=find_visual(tnew,new->TVscreen.visname,
				 &new->TVscreen.visual);
  v= new->TVscreen.visual;
#ifndef NOCARD
  if (new->TVscreen.bpp) new->TVscreen.wtw.bpp=new->TVscreen.bpp/8;
  else  new->TVscreen.wtw.bpp=tnew->core.depth/8;
  if (new->TVscreen.dual) new->TVscreen.wtw.bpp=4;
  new->TVscreen.wtw.bpl=rwidth*new->TVscreen.wtw.bpp;
  

  /* Alloc memory for snapshot */
  msize = 0x200000; /* largest possible picture*/
  if (!(tm=(unchar *)malloc(msize)))  {
    printf("Cannot alloc memory for snapshot\n");
    exit(1);
  }
  if (mlock(tm, msize))
    exit(1);
  
  /* The memory is formally allocated and locked but not really there yet!
     Access one byte in each page (assumed to be 4096 Bytes) to really do it.
     */
  for (ii=0; ii<msize; ii+=4096)
    tm[ii]=1;
  /* Just in case memory is not page aligned*/
  tm[msize-1]=1;
 
#ifdef DEBUG
  printf("tm: 0x%08x\n", (int) tm);
#endif
  new->TVscreen.TargetMem=tm;
  new->TVscreen.msize=msize;
  
#endif

  new->TVscreen.CMSET=False;
  if (v->class == PseudoColor){
    new->TVscreen.CMSET=True;
    my=new->TVscreen.colors;
    for (i=0;i<256;i++){
      my[i].pixel=i;
      my[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(disp, new->core.colormap, my, 256);

    cmap=XCreateColormap(disp, DefaultRootWindow(disp),v, AllocAll);
    new->core.colormap=cmap;
    for (i=0;i<225;i++){ /* Set Colors 16 - 240 */
      r=i%25;
      my[i+16].green =(65535*(i/25)/8);  
      my[i+16].red =(65535*(r/5)/4);  
      rr=r%5;
      my[i+16].blue =(65535*rr/4);        
    }
    XStoreColors(disp,cmap,my,256);
  }

  new->core.width=DEFAULT_WIDTH;
  new->core.height=DEFAULT_HEIGHT; 


  parent=tnew;
  while (parent->core.parent){
    parent=parent->core.parent;
  }

  XtAddEventHandler(tnew,SubstructureNotifyMask | VisibilityChangeMask
		    | StructureNotifyMask | ExposureMask
		    | EnterWindowMask |LeaveWindowMask,
		    FALSE,pConfig,new);
  XtAddEventHandler(parent,SubstructureNotifyMask | VisibilityChangeMask
		    | StructureNotifyMask | ExposureMask,
		    FALSE,pConfig,new);
  rootw=DefaultRootWindow(XtDisplay(tnew));
  XSelectInput(XtDisplay(tnew),rootw, VisibilityChangeMask |
	       SubstructureNotifyMask | ExposureMask |StructureNotifyMask);
  
  
  XtRegisterDrawable(XtDisplay(tnew),rootw,tnew);



  new->TVscreen.Change=FALSE;
  setmaxsize(new);
  new->core.background_pixel=BlackPixel(XtDisplay(new),
					DefaultScreen(XtDisplay(new)));

  timer=0;
  XSetErrorHandler((XErrorHandler) xerror);
}


static void SetGeo(cw)
TVscreenWidget cw;
{
#ifndef NOCARD
  Widget parent;
  struct bttv_window *wtw;
  Position x,y;
  int dx,dy;
  Boolean CHECK;
  int oldx,oldy,oldw,oldh;

  oldx=cw->TVscreen.wtw.x;
  oldy=cw->TVscreen.wtw.y;
  oldw=cw->TVscreen.wtw.width;
  oldh=cw->TVscreen.wtw.height;

  CHECK=cw->TVscreen.Change;

  if (cw->TVscreen.wtw.height<=(cw->TVscreen.wtw.cropheight/2))
    cw->TVscreen.wtw.interlace=0;
  else 
    cw->TVscreen.wtw.interlace=1;

  if (!XtIsRealized((Widget) cw)) return;

  parent=(Widget) cw;
  while(XtParent(parent)) parent=XtParent(parent);
  
  XtTranslateCoords((Widget)cw,0,0,&x,&y);
 
  if(oldh!=cw->core.height) CHECK=TRUE;
  if(oldw!=cw->core.width) CHECK=TRUE;

  dx=0;dy=0;
  if (cw->core.width > cw->TVscreen.max_width){
    dx=(cw->core.width- oldw)/2;
    if(oldw != cw->TVscreen.max_width){
      CHECK=TRUE;
      cw->TVscreen.wtw.width=cw->TVscreen.max_width & ~3;
    }
  } else {
    cw->TVscreen.wtw.width=cw->core.width & ~3;
  }

  if (cw->core.height > cw->TVscreen.max_height){
   dy=(cw->core.height- cw->TVscreen.wtw.height)/2;
    if(oldh != cw->TVscreen.max_height){
      CHECK=TRUE;
      cw->TVscreen.wtw.height=cw->TVscreen.max_height & ~3;
    }
  } else  cw->TVscreen.wtw.height=cw->core.height & ~3;

  if(oldx!=x+dx){
    Move=True;
    CHECK=True;
  }
  if(oldy!=y+dy){
    Move=True;
    CHECK=True;
  }

  if ((cw->TVscreen.wtw.bpp==3)&&((x+dx)&3)) {
    parent->core.x=parent->core.x-((x+dx) &3);
  } else if ((cw->TVscreen.wtw.bpp==2)&&((x+dx)&1)){
    parent->core.x=parent->core.x-((x+dx) &1);
  }
  if ((y+dy)&1) {
    parent->core.y=parent->core.y-((y+dy) &1);
  }
  
  if(CHECK)
    {
      cw->TVscreen.wtw.x=x+dx;
      cw->TVscreen.wtw.y=y+dy;
      if (cw->TVscreen.CAP_ON){
	ioctl(cw->TVscreen.fbttv, BTTV_SETWTW, &(cw->TVscreen.wtw));        
      }
    }  
  if(cw->TVscreen.CAP_ON &&  CHECK) getclip(cw); 
  if(!cw->TVscreen.Change && cw->TVscreen.CAP_ON && cw->TVscreen.Fresh) {
    refresh(cw);
    cw->TVscreen.Fresh=False;
  }
  cw->TVscreen.Change=FALSE;
#endif
}



static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
  Widget parent;
  TVscreenWidget curcw = (TVscreenWidget) current;
  TVscreenWidget newcw = (TVscreenWidget) new;
  Boolean do_redisplay = False;

#ifndef NOCARD

  if (newcw->TVscreen.CAP_ON){
    if (!curcw->TVscreen.CAP_ON) {
      do_redisplay =True;
      newcw->TVscreen.Change=TRUE;
      SetGeo(newcw);
    }
  }
  else {
    if (curcw->TVscreen.CAP_ON) {
      cap(newcw,0);
      system("xrefresh");
    }
    ioctl(newcw->TVscreen.fbttv, BTTV_INPUT, &(newcw->TVscreen.INPUT));
  }

  if (newcw->TVscreen.wtw.freq!=(ushort) newcw->TVscreen.FREQ){
    newcw->TVscreen.wtw.freq=(ushort) newcw->TVscreen.FREQ;
    ioctl(newcw->TVscreen.fbttv, BTTV_SETFREQ, &newcw->TVscreen.wtw.freq);
  }
  
  if (newcw->TVscreen.wtw.norm != newcw->TVscreen.NORM){
    newcw->TVscreen.wtw.norm=newcw->TVscreen.NORM;
    newcw->TVscreen.Change=TRUE;
    setmaxsize(newcw);
    SetGeo(newcw);
    if (newcw->TVscreen.NORM==NTSC) newcw->TVscreen.Fresh=TRUE;
    SetGeo(newcw);
  }
  if (curcw->TVscreen.INPUT !=newcw->TVscreen.INPUT){
    ioctl(newcw->TVscreen.fbttv, BTTV_INPUT, &(newcw->TVscreen.INPUT));
  }

  if (curcw->TVscreen.Bright !=newcw->TVscreen.Bright){
    ioctl(newcw->TVscreen.fbttv, BTTV_BRIGHT, &(newcw->TVscreen.Bright));
  }

  if (curcw->TVscreen.Color !=newcw->TVscreen.Color){
    ioctl(newcw->TVscreen.fbttv, BTTV_COLOR, &(newcw->TVscreen.Color));
  }

  if (curcw->TVscreen.Hue !=newcw->TVscreen.Hue){
    ioctl(newcw->TVscreen.fbttv, BTTV_HUE, &(newcw->TVscreen.Hue));
  }


  if (curcw->TVscreen.Contrast !=newcw->TVscreen.Contrast){
    ioctl(newcw->TVscreen.fbttv, BTTV_CONTRAST, &(newcw->TVscreen.Contrast));
  }
#endif

  return do_redisplay;
}







static void pConfig(wi,client_data,event)
     Widget wi;
     XtPointer client_data;
     XEvent *event;
{
  int fbttv;
  TVscreenWidget w=(TVscreenWidget) client_data;
  static Boolean UNMAPPED=False;
  XWindowAttributes wts1,wts2;
  Window checkw1,checkw2,wwin;
  Widget parent;

  if (!XtIsRealized((Widget)w)||(wwin=XtWindow((Widget)w)) == None) return;

  parent=(Widget)w;
  while (parent->core.parent){
    parent=parent->core.parent;
  }
  
  fbttv=w->TVscreen.fbttv;
  w->TVscreen.Fresh=True;
  cap(w,0);

  checkw1=event->xmap.window;
  checkw2=event->xany.window;
  if (w->TVscreen.CAP_ON)
    { 
      if ( wwin==checkw1 || wwin==checkw2 ) w->TVscreen.Fresh=False;
      else {
	if (checkw1){ 
	  XGetWindowAttributes(XtDisplay(w), checkw1, &wts1);
#ifndef NOCARD
	  if(!onscreen(w,wts1)) w->TVscreen.Fresh=False; 
#endif
	}
	if (checkw2) {
	  XGetWindowAttributes(XtDisplay(w), checkw2, &wts2);
#ifndef NOCARD
	  if(!onscreen(w,wts2)) w->TVscreen.Fresh=False; 
#endif
	}
      }
    }
  
  switch (event->type) {
  case VisibilityNotify:
    switch (event->xvisibility.state) {
    case VisibilityUnobscured:
      if (w->TVscreen.CAP_ON)
	{
	  if (checkw2 != DefaultRootWindow(XtDisplay(w))) {
	    w->TVscreen.Change=TRUE;
	    SetGeo(w);
	  }
	}
      break;
    case VisibilityFullyObscured:
      if (checkw2 != DefaultRootWindow(XtDisplay(w))) 
	cap(w,0);
      break;
    case VisibilityPartiallyObscured:
      if (w->TVscreen.CAP_ON)
	{
	  if (checkw2 != DefaultRootWindow(XtDisplay(w))) {
	    w->TVscreen.Change=TRUE;
	    SetGeo(w);
	  }
	}
      break;
    }
    break;

  case MapNotify:
    if ((DefaultRootWindow(XtDisplay(w)) != checkw2) && UNMAPPED){
      UNMAPPED=False;
      w->TVscreen.CAP_ON=True;
      cap(w,1);
      if ( w->TVscreen.Fresh ){
	refresh(w);
      }
    }
    if ((lastrwin != checkw1) && (w->TVscreen.CAP_ON) ){
      if ( w->TVscreen.Fresh ){
	refresh(w);
      }
    }
    break;


  case ConfigureNotify:
    cap(w,0);
    SetGeo(w);
    break;

  case DestroyNotify:
    if (wwin  == checkw2){
#ifdef DEBUG
      printf("Destroy\n");
#endif
      cap(w,0);
    }
    break;
    
  case UnmapNotify:
    if (DefaultRootWindow(XtDisplay(w)) != checkw2){
      cap(w,0);
      if (w->TVscreen.CAP_ON) UNMAPPED=True;
      if (w->TVscreen.CAP_ON) {
	if ( w->TVscreen.Fresh ){
	  refresh(w);
#ifdef DEBUG
    printf("UnmapNotify refresh\n");
#endif
	}
      }
      w->TVscreen.CAP_ON=False;
#ifdef DEBUG
      printf("Unmap\n");
#endif
    } else
    if (wwin  != event->xmap.window)
      if(w->TVscreen.CAP_ON) {
#ifndef NOCARD
	getclip(w); 
#endif
      }
    break;
  
  case EnterNotify:
    if (wwin  == checkw2){
      if (w->TVscreen.CAP_ON && w->TVscreen.CMSET){ 
	XInstallColormap(XtDisplay(w),w->core.colormap);
      }
    }
    break;

  case LeaveNotify:
/* Looks better this way */
/*    if (wwin  == checkw2){
      if (w->TVscreen.CAP_ON && w->TVscreen.CMSET){ 
	XUninstallColormap(XtDisplay(w),w->core.colormap);
      }
    }
*/ 
    break;
    
  }
  
  if(w->TVscreen.CAP_ON) {
	cap(w,1); 
      }
}

static XtGeometryResult QueryGeometry(w, proposed, answer)
     Widget w;
     XtWidgetGeometry *proposed, *answer;
{
  int reqm;
  TVscreenWidget cw = (TVscreenWidget) w;
  
  /* set fields we care about */
  reqm= CWWidth | CWHeight;
  answer->request_mode =reqm;
  
  /* initial width and height */
#ifndef NOCARD
  answer->width = (cw->TVscreen.wtw.width > 
		   cw->TVscreen.max_width) ? cw->TVscreen.max_width : 
		     cw->TVscreen.wtw.width;
  answer->height = (cw->TVscreen.wtw.height > 
		    cw->TVscreen.max_height) ? cw->TVscreen.max_height : 
		      cw->TVscreen.wtw.height;
  
  if (cw->TVscreen.wtw.bpp==3) {
    answer->width = answer->width & ~3; 
    answer->height = answer->height & ~3; 
  } else if (cw->TVscreen.wtw.bpp==2){
    answer->width = answer->width & ~1; 
    answer->height = answer->height & ~1; 
}
#else
    answer->width = DEFAULT_WIDTH; 
    answer->height = DEFAULT_HEIGHT; 

#endif


  if (  ((proposed->request_mode & (reqm)) == (reqm)) &&
      proposed->width == answer->width &&
      proposed->height == answer->height)
    return XtGeometryYes;
  else if (answer->width == cw->core.width &&
	   answer->height == cw->core.height)
    return XtGeometryNo;
  else
    return XtGeometryAlmost;
}

static void destroy(nw)
Widget nw;
{
  Widget parent;
  Window rootw;

  TVscreenWidget w=(TVscreenWidget) nw;

  w->TVscreen.CAP_ON=False;
  cap(w,0);
  if(w->TVscreen.CMSET){
    XFreeColormap(XtDisplay(w),w->core.colormap);
  }
  
  parent=nw;
  while (parent->core.parent){
    parent=parent->core.parent;
  }

  XtRemoveEventHandler(nw,SubstructureNotifyMask | VisibilityChangeMask
		    | StructureNotifyMask | ExposureMask
		    | EnterWindowMask |LeaveWindowMask,
		    FALSE,pConfig,w);
  XtRemoveEventHandler(parent,SubstructureNotifyMask | VisibilityChangeMask
		    | StructureNotifyMask | ExposureMask,
		    FALSE,pConfig,w);
  
  if (timer) XtRemoveTimeOut(timer);
  rootw=DefaultRootWindow(XtDisplay(nw));
  XtUnregisterDrawable(XtDisplay(nw),rootw);

#ifndef NOCARD
  munlock(w->TVscreen.TargetMem, w->TVscreen.msize);
  free(w->TVscreen.TargetMem);
#endif

  close(w->TVscreen.devname);
#ifdef DEBUG
  printf("Widget destroyed\n");
#endif

}
