/* 
    ViewWin Widget for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * ViewWin.c 
 */

#include <stdlib.h>
#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/IntrinsicP.h>
#include <X11/ObjectP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>

#include <stdio.h>

#include "ViewWinP.h"
#include "visual.h"

#define offset(field) XtOffsetOf(ViewWinRec, field)


static XtResource resources[] = {
  {
    XtNvis, 
    XtCvis, 
    XtRString, 
    sizeof(String),
    offset(ViewWin.visname), 
    XtRImmediate, 
    NULL
  },
};

/* Declaration of methods */

static void Initialize();
static void Realize();
static Boolean SetValues();

/* the following are actions of ViewWin */
static void pConfig();

ViewWinClassRec viewwinClassRec = {
    {
    /* core_class fields */
    /* superclass        */ (WidgetClass) &coreClassRec,
    /* class_name        */ "ViewWin",
    /* widget_size       */ sizeof(ViewWinRec),
    /* class_initialize      */ NULL,
    /* class_part_initialize     */ NULL,
    /* class_inited          */ FALSE,
    /* initialize        */ Initialize,
    /* initialize_hook       */ NULL,
    /* realize           */ Realize,
    /* actions           */ NULL,
    /* num_actions       */ 0,
    /* resources         */ resources,
    /* num_resources         */ XtNumber(resources),
    /* xrm_class         */ NULLQUARK,
    /* compress_motion       */ TRUE,
    /* compress_exposure     */ XtExposeCompressMultiple,
    /* compress_enterleave   */ TRUE,
    /* visible_interest      */ FALSE,
    /* destroy           */ destroy,
    /* resize            */ NULL,
    /* expose            */ NULL,
    /* set_values        */ SetValues,
    /* set_values_hook       */ NULL,
    /* set_values_almost     */ XtInheritSetValuesAlmost,
    /* get_values_hook       */ NULL,
    /* accept_focus      */ NULL,
    /* version           */ XtVersion,
    /* callback_private      */ NULL,
    /* tm_table          */ NULL,
    /* query_geometry        */ QueryGeometry,
    /* display_accelerator       */ XtInheritDisplayAccelerator,
    /* extension                 */ NULL
    },
    {
    /* extension          */        0,
    },
};

WidgetClass viewwinWidgetClass = (WidgetClass) & viewwinClassRec;

/* Realize */
static void Realize(Widget w,
		    XtValueMask *value_mask,
		    XSetWindowAttributes *attributes)
{
  ViewWinWidget tw=(ViewWinWidget) w;
    
  XtCreateWindow(w, (unsigned int) InputOutput,
		 tw->ViewWin.visual, *value_mask, attributes);

  if (tw->ViewWin.CAP_ON){
    tw->ViewWin.Change=TRUE;
    ioctl(tw->ViewWin.fbttv, BTTV_SETWTW, &(tw->ViewWin.wtw));     
    ioctl(tw->ViewWin.fbttv, BTTV_SETFREQ, &tw->ViewWin.wtw.freq);
    ioctl(tw->ViewWin.fbttv, BTTV_INPUT, &(tw->ViewWin.INPUT));
  }
} 

static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
  Widget parent;
  Screen *screen;
  Window rootw;
  int col;
  ViewWinWidget new = (ViewWinWidget) tnew;
  int major, minor, bank, ram;
  Display *disp;
  ushort i,r,rr;
  Colormap cmap;
  XColor *my;
  Visual *v;

  disp=XtDisplay(tnew);

  new->ViewWin.dual=find_visual(tnew,new->ViewWin.visname,
				 &new->ViewWin.visual);
  v= new->ViewWin.visual;

  if (v->class == PseudoColor){
    my=new->ViewWin.colors;
    for (i=0;i<256;i++){
      my[i].pixel=i;
      my[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(disp, new->core.colormap, my, 256);


    cmap=XCreateColormap(disp, DefaultRootWindow(disp),v, AllocAll);
    new->core.colormap=cmap;
    for (i=0;i<225;i++){ /* Set Colors 16 - 240 */
      r=i%25;
      my[i+16].green =(65535*(i/25)/8);  
      my[i+16].red =(65535*(r/5)/4);  
      rr=r%5;
      my[i+16].blue =(65535*rr/4);        
    }
    XStoreColors(disp,cmap,my,256);
  }


  XtAddEventHandler(tnew,EnterWindowMask |LeaveWindowMask,
		    FALSE,pConfig,new);
}



static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
  Widget parent;
  ViewWinWidget curcw = (ViewWinWidget) current;
  ViewWinWidget newcw = (ViewWinWidget) new;
  Boolean do_redisplay = False;

  return do_redisplay;
}


static void pConfig(wi,client_data,event)
     Widget wi;
     XtPointer client_data;
     XEvent *event;
{
  int fbttv;
  ViewWinWidget w=(ViewWinWidget) client_data;

  switch (event->type) {
  case EnterNotify:
    if (wwin  == checkw2){
      if (w->ViewWin.CAP_ON){ 
	XInstallColormap(XtDisplay(w),w->core.colormap);
      }
    }
    break;

  case LeaveNotify:
    if (wwin  == checkw2){
      if (w->ViewWin.CAP_ON){ 
	XUninstallColormap(XtDisplay(w),w->core.colormap);
      }
    }
    break;
  }
}
