/* 
 * Thomas Pantzer: capture.cc, v0.1b  28.1.98 18:6 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@aix520.informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */





//#include "bttv.h"
#include <stdio.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <fstream.h>

//#include <sys/socket.h>
// #include <X11/X.h>
// #include <X11/Xlib.h>

#include <forms.h>

#include "xcap.h"
#include "capture.h"
#include "utils.h"

#include <netinet/in.h>




t_sizemode SizeMode[] = {
                         {"160x120" , 1,160,120,1},
                         {"192x144" , 2,192,144,0},
                         {"240x180" , 3,240,180,1},
//                         {"288x216" , 4,288,216,0}, 
                         {"320x240" , 4,320,240,1},
                         {"360x288" , 5,360,288,0},
                         {"384x288" , 6,384,288,0},
//                         {"576x432" , 7,576,432,0}, 
                         {"640x480" , 8,640,480,1},
                         {"768x576" , 9,768,576,0},
                         {"16x9"    ,10,320,180,0}, 
                	 {"set mode", 0,768,576,0}
                        };
                         
#define VN(a) {a,#a}

Gmode gmode[] =
{
  VN(StaticGray), VN(GrayScale), VN(StaticColor),
  VN(PseudoColor), VN(TrueColor), VN(DirectColor),
};

#ifdef DEBUG
#define CHECK_POINT(x,y)	fprintf(stderr,"%s: p%d\n",x,y)
#else
#define CHECK_POINT(x,y)	
#endif

void grab_on(void)
{
   CHECK_POINT("grab_on",1);

   fl_set_idle_callback(idle_cb,0);
   dont_map_image = false;
  
   
   char msg[32];
   strcpy(msg,"get");
   write(tvsock_fd,&msg[0],strlen(msg)); 
}

void grab_off(void)
{
   CHECK_POINT("grab_off",1);

   dont_map_image = true;
   fl_set_idle_callback(nothing_cb,0);
}

void bttv_close(void)
{
   CHECK_POINT("client_close",1);

   shutdown(tvsock_fd,2);
      
   tvsock_fd = -1;   
}

void xcap_close(void)
{
   CHECK_POINT("xcap_close",1);
   
   grab_off();
   fl_hide_form(fd_xcap->xcap);
   bttv_close();
}


int GetModeInfo()
{
   int i,j;
   
   i = fl_state[fl_get_form_vclass(fd_xcap->xcap)].depth;
   j = (i / 8);
#ifdef DEBUG   
   fprintf(stderr,"visual depth = %d, return value = %d\n",i,j);
#endif
      
   return j;   
//   return 1;   //   8 Bit Display 
//   return 2;   //  16 Bit Display 
//   return 3;   //  24 Bit Display 
}


void SetColorMap(void)
{
   CHECK_POINT("SetColorMap",1);

   FL_COLOR cols[256];
       
//   for (int i=0; i<256; i++)
//     cols[i] = i;
         
//   fl_free_colors(cols,256);  

   int n = 16 + 15;
   for (int g=0; g<9; g++)
     {
      for (int r=0; r<5; r++)
        {
         for (int b=0; b<5; b++)
           {
            xcol[n].green = (65535 * g/8);  
            xcol[n].red   = (65535 * r/4);  
            xcol[n].blue  = (65535 * b/4);        
    
            xcol[n].pixel = n;
            xcol[n].flags = DoRed | DoGreen | DoBlue;

            n++;
           }
         }
      }
     
    cmap = XCreateColormap(disp,win,
                  DefaultVisual(disp,DefaultScreen(disp)),AllocAll);
    
#ifdef DEBUG   
    fprintf(stderr,"display 0x%lx, win 0x%lx, cmap 0x%lx\n",(long)disp,(long)win,cmap);
#endif
    XStoreColors(disp,cmap,xcol,256);
    fl_set_canvas_colormap(fd_xcap->canvas,cmap);
    fl_redraw_object(fd_xcap->canvas);    
}


void xcap_open(int bpp_mode,char *hostname_str,int portnum)
{
   if (tvsock_fd > 0)
      xcap_close();

   CHECK_POINT("xcap_open",1);   
   fl_show_form(fd_xcap->xcap,FL_PLACE_CENTER,FL_TRANSIENT,"xcap");

   CHECK_POINT("xcap_open",2);   
   fl_winresize(fl_get_canvas_id(fd_xcap->canvas),WIDTH,HEIGHT);   

   CHECK_POINT("xcap_open",3);   
   fl_winresize(fd_xcap->xcap->window,WIDTH+10,HEIGHT+10);   

   win = fl_get_canvas_id(fd_xcap->canvas);
   canvasGC = fl_state[fl_get_form_vclass(fd_xcap->xcap)].gc[0];

   CHECK_POINT("xcap_open",4);   


  struct hostent *HostName; 
  struct servent *MyService;
  
  struct sockaddr_in ServerAddr;

//Hostangabe aus 1. Parameter oder Umgebungsvariable
  HostName = gethostbyname(hostname_str);
    
  if (HostName == (struct hostent *) 0)
      {
        char ST[256];
        sprintf(ST,"gethostbyname(%s)",hostname_str);
        ErrorMsg(ST,errno);
        exit(2);
      }
  
//  printf("h_addr %d\n",HostName->h_addr);

//Portnummer aus 2. Parameter, /etc/services  oder vor-
//definiert aus "portnum.h"
  
  ServerAddr.sin_port = htons(portnum);
  ServerAddr.sin_family = AF_INET;
  ServerAddr.sin_addr.s_addr = *(unsigned long int *)HostName->h_addr;
  
  tvsock_fd = socket(AF_INET,SOCK_STREAM,0);
  if (tvsock_fd == -1)
   {
     ErrorMsg("socket()",errno);
     exit(3);
   }
   
  if (connect(tvsock_fd,(struct sockaddr *)&ServerAddr,sizeof(ServerAddr))==-1)
   {
//     printf(_CONNECT_ERR_STR,errno);
     ErrorMsg("connect()",errno); 
     exit(4);
   }

 
 /*
   tv_fd = open(BTTV_DEV,O_RDWR | O_EXCL | O_NDELAY);
   if (tv_fd <= 0) 
      {
       fprintf(stderr,"could not open device %s\n",BTTV_DEV);
       exit(1);
      }
       
   CHECK_POINT("xcap_open",5);   
   ioctl(tv_fd, BTTV_GETWTW, &wtw);
   wtw.x = 0;
   wtw.y = 0;
   wtw.bpp = bpp_mode;
   wtw.width = WIDTH;
   wtw.height = HEIGHT;
   wtw.cropwidth = WIDTH;
   wtw.cropheight = HEIGHT;
   wtw.cropx = 0;
   wtw.cropy = 0;
   wtw.swidth = WIDTH;
   wtw.sheight = HEIGHT;
   wtw.bpl = (WIDTH * wtw.bpp);
   vid_norm = pal;
   wtw.norm = vid_norm;
   
   if (WIDTH > 639)
      wtw.interlace = 1;
    else  
      wtw.interlace = 0;
      
//   wtw.color_fmt = 0x11;
   ioctl(tv_fd, BTTV_SETWTW, &wtw);
   ioctl(tv_fd, BTTV_INPUT, &vid_source);
  */
   x_img->width = WIDTH;
   x_img->height = HEIGHT;
   x_img->depth = 8 * bpp_mode;
   x_img->bytes_per_line = WIDTH * bpp_mode;
   x_img->bits_per_pixel = 8 * bpp_mode;
  
  
   switch(bpp_mode)
     {
      case 1:   /* 8 bit color cube 9x5x5 *
        x_img->red_mask   = 0x07;
        x_img->green_mask = 0x38;
        x_img->blue_mask  = 0xc0; */

        SetColorMap();
        break;
      
      case 2:   /* 16 bit Display */
#ifdef DEBUG
        fprintf(stderr,"16-bit XImage allocated\n");
#endif            
        x_img->red_mask   = 0xf800;
        x_img->green_mask = 0x07e0;
        x_img->blue_mask  = 0x001f;
        
        {
        int n = 16;
        for (int g=0; g<9; g++)
          for (int r=0; r<5; r++)
            for (int b=0; b<5; b++)
              {
               xcol[n].green = (63 * g/8);  
               xcol[n].red   = (63 * r/4);  
               xcol[n].blue  = (63 * b/4);        
    
//               xcol[n].pixel = n;
//               xcol[n].flags = DoRed | DoGreen | DoBlue;

               n++;
              }
        }
        break;

      case 3:   /* 24 bit Display */
        x_img->red_mask   = 0xff0000;
        x_img->green_mask = 0x00ff00;
        x_img->blue_mask  = 0x0000ff;
        {
        int n = 16;
        for (int g=0; g<9; g++)
          for (int r=0; r<5; r++)
            for (int b=0; b<5; b++)
              {
               xcol[n].green = (255 * g/8);  
               xcol[n].red   = (255 * r/4);  
               xcol[n].blue  = (255 * b/4);        
    
//               xcol[n].pixel = n;
//               xcol[n].flags = DoRed | DoGreen | DoBlue;

               n++;
              }
        }
        break;
     }   

#ifndef sun
  CHECK_POINT("xcap_open",6);   
  if (XInitImage(x_img) == 0)
     {
      fprintf(stderr,"XInitImage: error\n");
      bttv_close();
      exit(2);
     }
#endif     

   fcntl(tvsock_fd,F_SETFL, O_NONBLOCK); //fcntl(tvsock_fd,F_GETFL) & ~O_NONBLOCK);

/*
  CHECK_POINT("xcap_open",7);   
  int err = ioctl(tv_fd,BTTV_LOCK_XIMG,x_img); 
  if (err != 0)
     {
      fprintf(stderr,"could not announce ximage memory @ 0x%lx to driver (error %d)\n",(unsigned long)x_img,err);
      bttv_close();
      exit(3);
     }
 */
   grab_on();
}



void xcap_resize(int md)
{
   CHECK_POINT("xcap_resize",1);

/*
//   if (Size != md)
//      {
       xcap_close();
       xcap_open(wtw.bpp,SizeMode[md].width,
          	         SizeMode[md].height,
                         SizeMode[md].pal_mode);   
       Size = md;
      } 
 */      
} 


int xcap_init(void)
{
   CHECK_POINT("xcap_init",1);

   disp = FL_FormDisplay(fd_xcap->xcap);

   tvsock_fd = -1;
   dont_map_image = true;
   Size = 1;

   vid_norm = DEFAULT_NORM;
   vid_source = DEFAULT_SOURCE;   
   vid_std = DEFAULT_STD;

   pic_num = 1;
   seconds = 1;

#ifdef sun
   x_img = XCreateImage(disp,
        	        /*visual*/
                        fl_state[fl_get_form_vclass(fd_xcap->xcap)].xvinfo->visual, 
                        fl_state[fl_get_form_vclass(fd_xcap->xcap)].depth, 
                        ZPixmap,
                        0,
                        (char *)malloc(768*576*3),
                        768,
                        576,
                        32,
                        768*3);
#else
   x_img = (XImage *)malloc(sizeof(XImage));
   x_img->xoffset = 0;
   x_img->format = ZPixmap;

   x_img->data = (char *)malloc(768*576*3);

   x_img->bitmap_unit = 32; 
   x_img->bitmap_pad = 32;
   x_img->byte_order = LSBFirst;
   x_img->bitmap_bit_order = LSBFirst;
#endif
   

/*
   if (mlock(x_img->data, BTTV_LOCKMEMSIZE))
      {
       fprintf(stderr,"mlock failed\n");   
       exit(3);
      } 

   // swap the memory in 
   for (int i=0; i<BTTV_LOCKMEMSIZE; i += 4096)
      x_img->data[i] = 1;
 
   x_img->data[BTTV_LOCKMEMSIZE -1] = 1;
 */
   
//   xcap_open(SizeMode[Size].width,SizeMode[Size].height);
   return GetModeInfo();
}


void xcap_exit(void)
{
   CHECK_POINT("xcap_exit",1);

   xcap_close();
   
//   munlock(x_img->data,BTTV_LOCKMEMSIZE);
   free(x_img->data);
   
   exit(0);
}


void SetSource(int i)
{
   if (tvsock_fd < 0)
      return;
      
   vid_source = i % 4;
      
//   ioctl(tv_fd, BTTV_CAP_OFF, 0);
//   ioctl(tv_fd, BTTV_SETWTW, &wtw);
//   ioctl(tv_fd, BTTV_INPUT, &vid_source);
//   ioctl(tv_fd, BTTV_CAP_ON, 0);
   char msg[256];

   sprintf(msg,"source %d",vid_source);
   write(tvsock_fd,&msg[0],strlen(msg)); 

}


void SetFreq(double d)
{
   if (tvsock_fd < 0)
      return; 

   unsigned short freq = (unsigned short)(d * 16);
#ifdef DEBUG
   fprintf(stderr,"setting freq %d\n",freq);
#endif 

   char msg[256];

   sprintf(msg,"frequency %d",freq);
   write(tvsock_fd,&msg[0],strlen(msg)); 

//   ioctl(tv_fd, BTTV_CAP_OFF, 0);
/*
   ioctl(tv_fd, BTTV_SETWTW, &wtw);
   ioctl(tv_fd, BTTV_INPUT,  &vid_source);
   ioctl(tv_fd, BTTV_SETFREQ,&freq);
 */
//   ioctl(tv_fd, BTTV_CAP_ON, 0);
}




void SetCtrl(long c, double d)
{
   if (tvsock_fd < 0)
      return;   

   int i = (int)d;
   char msg[256];


   switch(c)
     {
      case 0: // brightness
//        ioctl(tv_fd,BTTV_BRIGHT,&i);
        sprintf(msg,"brite %d",i);
        write(tvsock_fd,&msg[0],strlen(msg)); 
        break;
        
      case 1: // hue
//        ioctl(tv_fd,BTTV_HUE,&i);
        sprintf(msg,"hue %d",i);
        write(tvsock_fd,&msg[0],strlen(msg)); 
        break;
        
      case 2: // color
//        ioctl(tv_fd,BTTV_COLOR,&i);
        sprintf(msg,"color %d",i);
        write(tvsock_fd,&msg[0],strlen(msg)); 
        break;
        
      case 3: // contrast
//        ioctl(tv_fd,BTTV_CONTRAST,&i);
        sprintf(msg,"contrast %d",i);
        write(tvsock_fd,&msg[0],strlen(msg)); 
        break;        
     }
}




int error_cap = 0;
int RD = 0;
int block_flag = 0;
char resend_spawned = false;


/*
int bit_mirror(int arg,int width)
{
   int res,msk,set,hi;
   
   res = 0;
   hi = 1 << (width-1);
   for (int i=0; i<width; i++)
     {
      msk = 1 << i;
      set = hi >> i;
      
      if ((msk & arg) > 0)
          res |= set;
     }
     
   return res;  
}
 */

void Do_Capture(void)
{
   if (
       (dont_map_image == true) || 
       (tvsock_fd < 0) ||
       (fd_xcap->xcap->visible == false)
      )
      return;

   int err;

/*
   if (ioctl(tv_fd,BTTV_XIMG_READY,0) == 0)
      {
       if (idle_cnt > 20)
          err = ioctl(tv_fd,BTTV_FILL_XIMG,0); 
      
       
       if (error_cap > 96)
          {
           xcap_resize(Size); 
           error_cap = 0;
          } 
           
          
       error_cap++;    
#ifdef DEBUG
       fprintf(stderr,"seconds %d, error_cap %d\n",seconds,error_cap); 
#endif              
       return;
      } 
 */
//   unsigned int *Buf32;
   unsigned short *Buffer;
   unsigned char buf[(WIDTH*HEIGHT*BPP*2) + 5];
//   unsigned short Buffer[WIDTH*HEIGHT];
   
   char msg[32];
   int rd,tr;
      
   if (FD_Ready(tvsock_fd))
      {
       tr = PIC_SIZE - RD;
//       CHECK_POINT("data to get",tr);


       rd = read(tvsock_fd,(char *)&buf[RD],tr);

       if (rd <= 0)
          {
           switch (errno)
             {
              case 11:  //EWOULDBLOCK
                block_flag++;   

                if (((block_flag > 8) && (resend_spawned == false)) || (block_flag > 64))
                   {
                    CHECK_POINT("would block resend",RD);
                 
                    sprintf(msg,"resend %d",RD);
                    write(tvsock_fd,&msg[0],strlen(msg)); 
                    block_flag = 0;
                    resend_spawned = true;
                   } 
                break;
      
              case 0:
              case 89:  // function not implemented  (conn.reset by peer on sun ??)
              case 131: // connection reset by peer
                xcap_exit();
                exit(1);
                break;
                        
              default:
                ErrorMsg("read_socket()",errno);
                
//                if ((RD == 0) || (idle_cnt > 23))
//                   {
//                    CHECK_POINT("..data",RD);
//                    CHECK_POINT("..idle_cnt",idle_cnt);
//                    strcpy(msg,"get");
//                    write(tvsock_fd,&msg[0],strlen(msg)); 
//                   } 
                break;
             }
              
           return;
          } 
        else           
          {
           block_flag = 0;
           resend_spawned = false;
       
           if (strncmp(&buf[RD],"start",5) == 0) 
              {
               int rs = rd - 5;
               
               for (int i=0; i<rs; i++)
                  buf[i] = buf[RD+5+i];
                  
               RD = rs;   
               
//               CHECK_POINT("sync read",rd);
              }
            else
              {
//               for (int i=0; i<rd; i++)
//                 buf[RD+i] = msg[i];
               
               RD += rd;  
//               CHECK_POINT("data read",rd);
              } 
          } 
       
       if (RD < PIC_SIZE)
          return;
      } 
    else
      {
//       block_flag++;

//       if (((block_flag  > 8) && (resend_spawned == false)) || (block_flag > 64))
//          {
//           CHECK_POINT("not ready get sync",RD);
//           sprintf(msg,"sync");
//           write(tvsock_fd,&msg[0],strlen(msg)); 
//           block_flag = 0;
//           resend_spawned = true;
//          } 

       return;   
      } 
   
//   CHECK_POINT("got picture",RD);
   RD = 0;
   
   switch (x_img->depth)
     {
      case  8:  // case visual 0(mono)  1(mapped color) 4(true color) 
        for (int i=0; i<PIC_SIZE; i++)
          x_img->data[i] = buf[i] + 15;
        break;
        
      case 16: // 8 -> 16
        Buffer = (unsigned short *)&x_img->data[0];               
        
        for (int h=0; h<HEIGHT; h++)
           for (int w=0; w<WIDTH; w++)
             {
              int i,j;
              unsigned long r,g,b,l;
              
              i = w + (h*WIDTH);
              j = buf[i];
              
              r = xcol[j].red;   r = r << 10; r &= 0xf800;
              g = xcol[j].green; g = g <<  5; g &= 0x07e0;
              b = xcol[j].blue;  b = b >>  1; b &= 0x001f;
              
              l = (r | g | b);
              
              Buffer[i] = l; //((l >> 8) & 0xff) | ((l << 8) & 0xff00);
             }
        break;  

      case 24:   // 8 -> 24
        for (int h=0; h<HEIGHT; h++)
           for (int w=0; w<WIDTH; w++)
             {
              int i,j,k;
             // unsigned long r,g,b,l;
              
              i = w + (h*WIDTH);
              j = buf[i];
              k = i*3;
              
//              Buf32 = &x_img->data[k];
              
              x_img->data[k]   = xcol[j].red;   
              x_img->data[k+1] = xcol[j].green;
              x_img->data[k+2] = xcol[j].blue; 
             }
        break;
        
 
      case 88: // 16 -> 8  
        for (int h=0; h<HEIGHT; h++)
          for (int w=0; w<WIDTH; w++)              
            {
             int i = (h*WIDTH) + w;
             int j = i*2;
             int k;

             k = (buf[j+1] * 256) + buf[j];
             
             unsigned int r,g,b;
             
             r = ((k & 0xf800) >> 11) & 0x1f;
             g = ((k & 0x07e0) >>  5) & 0x3f;
             b =  (k & 0x001f);


             unsigned int bi,ri,gi;
             double d,rf,gf,bf;

             switch (visual_mode)
               {
                case 0: // gray levels
                  d = ((double)r * 0.3 / 32.0)  + 
                      ((double)g * 0.59 / 64.0) + 
                      ((double)b * 0.11 / 32.0);     
  
             	  x_img->data[i] = (unsigned char)(d * 255.0);
                  break;
                  
                default:
                case 1: // 9x5x5 GRB color cube
                  rf = (double)r * 5.0 / 32.0;
                  ri = (unsigned int)rf;
                  
                  gf = (double)g * 9.0 / 64.0;
                  gi = (unsigned int)gf;
                 
                  bf = (double)b * 5.0 / 32.0;
                  bi = (unsigned int)bf;
                                       
                  x_img->data[i] = 15 + 16 + bi + (ri * 5) + (gi * 25);
                  break;
                  
                case 4: // 8bit true color
                  ri = (r *  8);
                  ri /= 0x20; 
             
                  gi = (g *  8);
                  gi /= 0x40;
             
                  bi = (b *  4);
                  bi /= 0x20;
           
                  x_img->data[i] = (ri & 0x07) + ((gi << 3) & 0x38)  + ((bi << 6) & 0xc0);
                  break;
               }

  /*
  */
  /*
   */          
            }
        break;
        
//      case 16: // 16 -> 16
//        break;
        
//      case 24:  // 16 -> 24
//        break;  
     }
         
#ifdef COLORBAR
   if (x_img->depth == 8)
      for (int j=0; j<8; j++)     
        for (int i=0; i<256; i++)
           x_img->data[i+(j*x_img->width)] = i;
#endif
       
   XPutImage(disp,win,canvasGC,x_img,0,0,0,0,x_img->width,x_img->height);
   
   error_cap = 0;
   pic_num++;

   if ((pic_num % 64) == 0)
      {
       double d;
       d = (double)pic_num;

       if (seconds > 0)
          FPS = d / (double)seconds;
       
       pic_num = 0;
       seconds = 0;
#ifdef DEBUG       
       fprintf(stdout,"%3.1f frames per second\n",FPS);       
#endif
      } 

/*      
  //discard
  //
  tr = read(tvsock_fd,(char *)&buf[0],PIC_SIZE);
  rd = tr;
  while (tr > 0)
    {
     tr = read(tvsock_fd,(char *)&buf[0],PIC_SIZE);
     rd += tr;
    }
      
  CHECK_POINT("discard data",rd);
  usleep(200000);
  */ 
  strcpy(msg,"sync");
  write(tvsock_fd,&msg[0],strlen(msg)); 
/*
   err = ioctl(tv_fd,BTTV_FILL_XIMG,0); 
   if (err != 0)
      {
       fprintf(stderr,"could not grab error %d\n",err);
       return; 
      } 
 */

}




// #define SEND(x)		{ char msg[32]; strcpy(msg,x); write(tvsock_fd,&msg[0],strlen(msg));}
