/* 
 * Thomas Pantzer: netutils.cc, v0.1b  28.1.98 18:6 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@aix520.informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */




#include "netutils.h"
#include "utils.h"



void hostname_str(struct sockaddr_in *Client,char *result)
{
  unsigned long int i,j;
  unsigned char k,l,m,n;
  char st[256],st2[256];
  struct hostent *remote;

  i=Client->sin_addr.s_addr / 65536;
  j=Client->sin_addr.s_addr % 65536;
  k=i / 256;
  l=i % 256;
  m=j / 256;
  n=j % 256;
  
//  sprintf(result,"%d.%d.%d.%d",n,m,l,k);

#if (sun || hpux)
  sprintf(st,"%d.%d.%d.%d",k,l,m,n);
  sprintf(st2,"%c%c%c%c",k,l,m,n);
#else
  sprintf(st,"%d.%d.%d.%d",n,m,l,k);
  sprintf(st2,"%c%c%c%c",n,m,l,k);
#endif    

#ifdef DEBUG  
  fprintf(stderr,"remote address: %s\n",st); 
  fprintf(stderr,"remote port: %d\n",Client->sin_port);
#endif  

  remote = gethostbyaddr(st2,strlen(st2),AF_INET); 
  if (remote == NULL)
     {
      ErrorMsg("gethostbyaddr");
      sprintf(result,"%s:%d",st,Client->sin_port);
     } 
   else  
     {
#ifdef DEBUG
      fprintf(stderr,"remote name: %s\n",remote->h_name);
#endif      
      sprintf(result,"%s:%d",remote->h_name,Client->sin_port);
     } 
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// zum bestimmen der Portnummer aus /etc/services (falls vorhanden)
//________________________________________________________________
int GetTCPPort(char proto[64])
{
  struct servent *MyService;

  MyService = getservbyname(proto,"tcp");

  if (MyService != NULL)
       {
        printf("using port %d from default service for %s\n",
                  ntohs(MyService->s_port),MyService->s_name);
        return ntohs(MyService->s_port);  
       } 
     else 
       return _MY_PORTNUM;
}  

int clConnect_to(char *hname, int port_num)
{
  struct hostent *HostName; 

  struct sockaddr_in ServerAddr;
  int _Sock;
  char ST[256];

  HostName = gethostbyname(hname);
 
  if (HostName == (struct hostent *) 0)
      {
        printf("gethostbyname(%s): failed\b\n",ST);
        exit(2);
      }
  
  ServerAddr.sin_port=htons(port_num);
  ServerAddr.sin_family=AF_INET;
  ServerAddr.sin_addr.s_addr=*(unsigned long int *)HostName->h_addr;
  
  _Sock=socket(AF_INET,SOCK_STREAM,0);
  if (_Sock==-1)
   {
     ErrorMsg("socket()");
     exit(3);
   }
   
  if (connect(_Sock,(struct sockaddr *)&ServerAddr,sizeof(ServerAddr))==-1)
   {
     ErrorMsg("connect()",errno); 
     exit(4);
   }

#ifdef DEBUG
  fprintf(stderr,"assigning socket=%d\n",_Sock);
#endif   
  return _Sock; 
}


void ShutDown(int sd)
{
   struct linger L;

   fcntl(sd,F_SETFL,O_NONBLOCK);  // get it faster to shut up

   L.l_onoff = 1;
   L.l_linger = LINGER_TIMEOUT;    
    
#if (linux || hpux)  
   if (setsockopt(sd,SOL_SOCKET,SO_LINGER,&L,sizeof(struct linger)) == -1)
#else
#ifdef sun
   if (setsockopt(sd,SOL_SOCKET,SO_LINGER,
          (char *)&L,sizeof(struct linger)) == -1)
#else
   make ugly errors here 
#endif	  
#endif   
      ErrorMsg("setsockopt()");  

   shutdown(sd,2);
}


int svAccept(int Sock)
{
   int peer;
   int CNameLen;
   struct sockaddr_in ClientAddr;
   char st[256];

    
   CNameLen = sizeof(sockaddr_in);
   peer = accept(Sock,(struct sockaddr *)&ClientAddr,&CNameLen);
   
   hostname_str(&ClientAddr,&st[0]);
   fprintf(stderr,"player connected from %s\n",st);

   return peer;
}


int svListen(int port)
{
   int L;
   struct sockaddr_in ServerAddr;

   ServerAddr.sin_family = AF_INET;
   ServerAddr.sin_port = htons(port);
   ServerAddr.sin_addr.s_addr = INADDR_ANY;

   L = socket(AF_INET,SOCK_STREAM,0);
   if (L == -1)
      {
       ErrorMsg("socket()");
       exit(2);
      } 

   if (bind(L,(struct sockaddr *)&ServerAddr,sizeof(ServerAddr))==-1)
      {
       ErrorMsg("bind()");
       exit(3); 
      }
  
   if (listen(L,1) == -1)
      {
       ErrorMsg("listen()");
       exit(4);
      }

   
   return L;
}



