/* 
 * Thomas Pantzer: utils.cc, v0.1b  21.9.97 22:58 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@aix520.informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */


#include "utils.h"



// Kommentar eigentlich total berflssig
int max(int a, int b)
{
  if (a>b)
     return a;
   else 
     return b;
}       

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ist wahr, wenn es aus der Datei des Filedescriptors 'was zu lesen gibt  
// gewartet wird ca. 10ms
//______________________________________________________________________
int FD_Ready(int fd)
{
  struct fd_set Read_Set;
  struct timeval TimeOut;

//  FD_ZERO(&Read_Set);
  FD_SET(fd,&Read_Set);
  TimeOut.tv_sec=0;
  TimeOut.tv_usec=10000;

#ifdef hpux
  select(fd+1,(int *)&Read_Set,0,0,&TimeOut);
#else
  select(fd+1,&Read_Set,0,0,&TimeOut);
#endif    
  if FD_ISSET(fd,&Read_Set)
       return 1;
    else
       return 0;   
}



void ErrorMsg(const char st[])
{
   ErrorMsg(st,errno);
}

void ErrorMsg(const char st[],int no)
{
  fprintf(stderr,"%s:%d\t",st,no);

  switch(no)
  {
	case	EWOULDBLOCK 	: fprintf(stderr," Operation would block	"); break;
   	case	EDOM		: fprintf(stderr," Math arg out of domain of func "); break;
	case	ERANGE		: fprintf(stderr," Math result not representable "); break;
        case	EPERM		: fprintf(stderr," Not super-user		"); break;
	case	ENOENT		: fprintf(stderr," No such file or directory	"); break;
	case	ESRCH		: fprintf(stderr," No such process		"); break;
	case	EINTR		: fprintf(stderr," interrupted system call	"); break;
	case	EIO		: fprintf(stderr," I/O error			"); break;
	case	ENXIO		: fprintf(stderr," No such device or address	"); break;
	case	E2BIG		: fprintf(stderr," Arg list too long		"); break;
	case	ENOEXEC		: fprintf(stderr," Exec format error		"); break;
	case	EBADF		: fprintf(stderr," Bad file number		"); break;
	case	ECHILD		: fprintf(stderr," No children			"); break;
	case	ENOMEM		: fprintf(stderr," Not enough core		"); break;
	case	EACCES		: fprintf(stderr," Permission denied		"); break;
	case	EFAULT		: fprintf(stderr," Bad address			"); break;
	case	EBUSY		: fprintf(stderr," Mount device busy		"); break;
	case	EEXIST		: fprintf(stderr," File exists			"); break;
	case	EXDEV		: fprintf(stderr," Cross-device link		"); break;
	case	ENODEV		: fprintf(stderr," No such device		"); break;
	case	ENOTDIR		: fprintf(stderr," Not a directory		"); break;
	case	EISDIR		: fprintf(stderr," Is a directory		"); break;
	case	EINVAL		: fprintf(stderr," Invalid argument		"); break;
	case	ENFILE		: fprintf(stderr," File table overflow		"); break;
	case	EMFILE		: fprintf(stderr," Too many open files		"); break;
	case	ENOTTY		: fprintf(stderr," Not a typewriter		"); break;
	case	EFBIG		: fprintf(stderr," File too large		"); break;
	case	ENOSPC		: fprintf(stderr," No space left on device	"); break;
	case	ESPIPE		: fprintf(stderr," Illegal seek			"); break;
	case	EROFS		: fprintf(stderr," Read only file system	"); break;
	case	EMLINK		: fprintf(stderr," Too many links		"); break;
	case	EPIPE		: fprintf(stderr," Broken pipe			"); break;
	case	EDEADLK 	: fprintf(stderr," A deadlock would occur	"); break;
	case	ENOLCK 		: fprintf(stderr," System record lock table was full "); break;
	case	ENOTEMPTY  	: fprintf(stderr," Directory not empty     "); break;
	case	ENAMETOOLONG 	: fprintf(stderr," File name too long      "); break;
	case	ENOSYS 	 	: fprintf(stderr," Function not implemented   "); break;
	case	ENOTBLK		: fprintf(stderr," Block device required	"); break;
	case	ETXTBSY		: fprintf(stderr," Text file busy		"); break;
	case	ENOMSG 	   	: fprintf(stderr," No message of desired type  "); break;
	case	EIDRM		: fprintf(stderr," Identifier removed		"); break;
	case	ELOOP	  	: fprintf(stderr," Too many levels of symbolic links "); break;

        
#ifndef sun        
	case	EDOTDOT 	: fprintf(stderr," Cross mount point (not really error)"); break;
#endif

#if !(hpux || sun)
        case	EILSEQ		: fprintf(stderr," Illegal byte sequence    "); break;
	case	ELNRNG		: fprintf(stderr," Link number out of range		"); break;
	case	EUNATCH 	: fprintf(stderr," Protocol driver not attached		"); break;
	case	ENOCSI		: fprintf(stderr," No CSI structure available		"); break;
	case	ECHRNG		: fprintf(stderr," Channel number out of range		"); break;
	case	EL2NSYNC 	: fprintf(stderr," Level 2 not synchronized		"); break;
	case	EL3HLT		: fprintf(stderr," Level 3 halted			"); break;
	case	EL3RST		: fprintf(stderr," Level 3 reset			"); break;
	case	EL2HLT		: fprintf(stderr," Level 2 halted			"); break;
	case	ENOPKG		: fprintf(stderr," Package not installed        "); break;
	case	ENODATA		: fprintf(stderr," no data (for no delay io)		"); break;

#endif        

#if !(linux || sun)
// and this could be a duplicate case value         
	case	EAGAIN		: fprintf(stderr," No more processes		"); break;
	case	ENOSYM		: fprintf(stderr," symbol does not exist in executable "); break;
	case	EREMOTERELEASE	: fprintf(stderr," Remote peer released connection "); break;
	case 	EUNEXPECT	: fprintf(stderr," Unexpected Error		"); break;
	case	ENOMSG   	: fprintf(stderr," No message of desired type "); break;
#endif

        
	case	ENONET		: fprintf(stderr," Machine is not on the network	"); break;
	case	ETIME		: fprintf(stderr," timer expired			"); break;
	case	ENOSR		: fprintf(stderr," out of streams resources		"); break;
	case	ENOSTR		: fprintf(stderr," Device not a stream			"); break;
	case	ENOLINK		: fprintf(stderr," the link has been severed "); break;
	case	EADV		: fprintf(stderr," advertise error "); break;
	case	ESRMNT		: fprintf(stderr," srmount error "); break;
	case	ECOMM		: fprintf(stderr," Communication error on send		"); break;
	case	EPROTO		: fprintf(stderr," Protocol error			"); break;
	case	EMULTIHOP 	: fprintf(stderr," multihop attempted "); break;
	case	EBADMSG 	: fprintf(stderr," trying to read unreadable message	"); break;
	case	EUSERS	 	: fprintf(stderr," For Sun compatibilty, will not occur."); break;
	case	EDQUOT	 	: fprintf(stderr," Disc quota exceeded 		"); break;
	case	ESTALE		: fprintf(stderr," Stale NFS file handle "); break;
	case	EREMOTE		: fprintf(stderr," Too many levels of remote in path "); break;

	case	ENOTSOCK	: fprintf(stderr," Socket operation on non-socket "); break;
	case	EDESTADDRREQ	: fprintf(stderr," Destination address required "); break;
	case	EMSGSIZE	: fprintf(stderr," Message too long "); break;
	case	EPROTOTYPE	: fprintf(stderr," Protocol wrong type for socket "); break;
	case	ENOPROTOOPT	: fprintf(stderr," Protocol not available "); break;
	case	EPROTONOSUPPORT	: fprintf(stderr," Protocol not supported "); break;
	case	ESOCKTNOSUPPORT	: fprintf(stderr," Socket type not supported "); break;
	case	EOPNOTSUPP	: fprintf(stderr," Operation not supported "); break;
	case	EPFNOSUPPORT 	: fprintf(stderr," Protocol family not supported "); break;
	case	EAFNOSUPPORT 	: fprintf(stderr," Address family not supported by protocol family"); break;
	case	EADDRINUSE	: fprintf(stderr," Address already in use "); break;
	case	EADDRNOTAVAIL 	: fprintf(stderr," Can't assign requested address "); break;
	case	ENETDOWN	: fprintf(stderr," Network is down "); break;
	case	ENETUNREACH	: fprintf(stderr," Network is unreachable "); break;
	case	ENETRESET	: fprintf(stderr," Network dropped connection on reset "); break;
	case	ECONNABORTED	: fprintf(stderr," Software caused connection abort "); break;
	case	ECONNRESET	: fprintf(stderr," Connection reset by peer "); break;
	case	ENOBUFS		: fprintf(stderr," No buffer space available "); break;
	case	EISCONN		: fprintf(stderr," Socket is already connected "); break;
	case	ENOTCONN	: fprintf(stderr," Socket is not connected "); break;
	case	ESHUTDOWN	: fprintf(stderr," Can't send after socket shutdown "); break;
	case	ETOOMANYREFS	: fprintf(stderr," Too many references: can't splice "); break;
	case	ETIMEDOUT	: fprintf(stderr," Connection timed out "); break;
	case	ECONNREFUSED	: fprintf(stderr," Connection refused "); break;
	case	EHOSTDOWN	: fprintf(stderr," Host is down "); break;
	case	EHOSTUNREACH	: fprintf(stderr," No route to host "); break;
	case	EALREADY  	: fprintf(stderr," Operation already in progress "); break;
	case	EINPROGRESS 	: fprintf(stderr," Operation now in progress "); break;
  }
  
  fprintf(stderr,"\n");
}





